/***************************************************************
 * Name:      wx015Main.cpp
 * Purpose:   Code for Application Frame
 * Author:    Salvador Pozo (salvador@conclase.net)
 * Created:   2025-01-01
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#include <ctime>
#include "wx015Main.h"
#include "red.xpm"
#include "green.xpm"

//helper functions
enum wxbuildinfoformat {
    short_f, long_f };

wxString wxbuildinfo(wxbuildinfoformat format)
{
    wxString wxbuild(wxVERSION_STRING);

    if (format == long_f )
    {
#if defined(__WXMSW__)
        wxbuild << _T("-Windows");
#elif defined(__WXMAC__)
        wxbuild << _T("-Mac");
#elif defined(__UNIX__)
        wxbuild << _T("-Linux");
#endif

#if wxUSE_UNICODE
        wxbuild << _T("-Unicode build");
#else
        wxbuild << _T("-ANSI build");
#endif // wxUSE_UNICODE
    }

    return wxbuild;
}

BEGIN_EVENT_TABLE(wx015Frame, wxFrame)
    EVT_CLOSE(wx015Frame::OnClose)
    EVT_SIZE(wx015Frame::OnSize)
    EVT_MENU(idMenuQuit, wx015Frame::OnQuit)
    EVT_MENU(idMenuAbout, wx015Frame::OnAbout)
    EVT_MENU(idMenuRojo, wx015Frame::OnRojo)
    EVT_MENU(idMenuVerde, wx015Frame::OnVerde)
    EVT_BUTTON(idBoton, wx015Frame::OnBoton)
END_EVENT_TABLE()

wx015Frame::wx015Frame(wxFrame *frame, const wxString& title)
    : wxFrame(frame, -1, title), sb(nullptr), color(verde)
{
    // create a menu bar
    wxMenuBar* mbar = new wxMenuBar();
    wxMenu* fileMenu = new wxMenu(_T(""));
    fileMenu->Append(idMenuQuit, _T("&Salir\tAlt-F4"), _T("Abandonar la aplicación"));
    mbar->Append(fileMenu, _T("&Fichero"));

    wxMenu* colorMenu = new wxMenu(_T(""));
    colorMenu->Append(idMenuRojo, _T("&Rojo"), _T("Cambia bitmap a rojo"));
    colorMenu->Append(idMenuVerde, _T("&Verde"), _T("Cambia bitmap a verde"));
    mbar->Append(colorMenu, _("&Color"));

    wxMenu* helpMenu = new wxMenu(_T(""));
    helpMenu->Append(idMenuAbout, _T("&Sobre\tF1"), _T("Muestra información sobre la aplicación"));
    mbar->Append(helpMenu, _("&Ayuda"));

    SetMenuBar(mbar);

    // create a status bar with some information about the used wxWidgets version
    sb = new wxStatusBar(this);
    sb->SetFieldsCount(5);
    int width[] = {-1, 120, 70, 50, 8};
    int style[] = {wxSB_RAISED, wxSB_NORMAL, wxSB_NORMAL, wxSB_SUNKEN, wxSB_FLAT};
    wxRect rect;
    sb->SetStatusWidths(5, width);
    sb->SetStatusStyles(5, style);
    sb->SetStatusText(_("Hola"),0);
    sb->SetStatusText(wxbuildinfo(short_f), 1);
    SetStatusBar(sb);
    sb->GetFieldRect(zonaColor, rect);
    bm = new wxStaticBitmap(sb, wxID_ANY, wxBitmapBundle(verde_xpm), wxPoint(rect.x+3, rect.y+3));
    sb->GetFieldRect(zonaBoton, rect);
    bt = new wxButton(sb, idBoton, _T("Cambio"), wxPoint(rect.x, rect.y), wxSize(rect.GetWidth(), rect.GetHeight()));
    sb->GetFieldRect(zonaHora, rect);
    sb->SetStatusText(_T("00:00:00"), 2);
    hilo = std::thread(&Hora, this);
}

wx015Frame::~wx015Frame()
{
    hilo.detach();
    delete bt;
    delete bm;
    delete sb;
}

void wx015Frame::Hora(wx015Frame* frame) {
    static wxString hora(_("00:00:00"));
    std::time_t t_c=0;
    std::tm tiempo;

    do {
        t_c = std::time(nullptr);
        tiempo = *std::localtime(&t_c);
        hora.Printf(_("%02d:%02d:%02d"), tiempo.tm_hour, tiempo.tm_min, tiempo.tm_sec);
        frame->sb->SetStatusText(hora, 2);
        std::this_thread::sleep_for(std::chrono::milliseconds(250));
    } while(true);
}

void wx015Frame::OnSize(wxSizeEvent& event) {
    if(!sb) return;
    wxRect rect;
    sb->GetFieldRect(zonaColor, rect);
    bm->Move(wxPoint(rect.x+3, rect.y+3));
    sb->GetFieldRect(zonaBoton, rect);
    bt->Move(wxPoint(rect.x, rect.y));
    sb->GetFieldRect(zonaHora, rect);
}

void wx015Frame::OnClose(wxCloseEvent &event)
{
    Destroy();
}

void wx015Frame::OnQuit(wxCommandEvent &event)
{
    Destroy();
}

void wx015Frame::OnAbout(wxCommandEvent &event)
{
    wxString msg = wxbuildinfo(long_f);
    wxMessageBox(msg, _("Bienvenido a..."));
}

void wx015Frame::OnVerde(wxCommandEvent& event)
{
    color = verde;
    ActualizaColor();
}

void wx015Frame::OnRojo(wxCommandEvent& event)
{
    color = rojo;
    ActualizaColor();
}

void wx015Frame::OnBoton(wxCommandEvent &event)
{
    if(color==verde) color=rojo;
    else color=verde;
    ActualizaColor();
}

void wx015Frame::ActualizaColor()
{
    if(color==verde)
        bm->SetBitmap(verde_xpm);
    else
        bm->SetBitmap(rojo_xpm);
}
